@@ -1,3 +1,10 @@
+## Copyright (c) 2009-2011, Noel O'Boyle
+## All rights reserved.
+##
+##  This file is part of Cinfony.
+##  The contents are covered by the terms of the GPL v2 license
+##  which is included in the file LICENSE_GPLv2.txt.
+
 """
 ironable - A Cinfony module for accessing OpenBabel from IronPython
 
@@ -13,11 +20,22 @@ Global variables:
 import os
 import math
 import os.path
+import tempfile
 
 import System
 import clr
-obdotnet = os.environ["OBDOTNET"]
-clr.AddReferenceToFileAndPath(obdotnet + "\\OBDotNet.dll")
+clr.AddReference('System.Windows.Forms')
+clr.AddReference('System.Drawing')
+ 
+from System.Windows.Forms import (
+    Application, DockStyle, Form, PictureBox, PictureBoxSizeMode
+    )
+from System.Drawing import Image, Size
+
+_obdotnet = os.environ["OBDOTNET"]
+if _obdotnet[0] == '"': # Remove trailing quotes
+    _obdotnet = obdotnet[1:-1]
+clr.AddReferenceToFileAndPath(os.path.join(_obdotnet, "OBDotNet.dll"))
 import OpenBabel as ob
 
 def _formatstodict(list):
@@ -82,12 +100,14 @@ def readfile(format, filename):
         raise ValueError("%s is not a recognised OpenBabel format" % format)
     if not os.path.isfile(filename):
         raise IOError("No such file: '%s'" % filename)
-    obmol = ob.OBMol()
-    notatend = obconversion.ReadFile(obmol,filename)
-    while notatend:
-        yield Molecule(obmol)
+    def filereader():
         obmol = ob.OBMol()
-        notatend = obconversion.Read(obmol)
+        notatend = obconversion.ReadFile(obmol,filename)
+        while notatend:
+            yield Molecule(obmol)
+            obmol = ob.OBMol()
+            notatend = obconversion.Read(obmol)
+    return filereader()
 
 def readstring(format, string):
     """Read in a molecule from a string.
@@ -228,7 +248,7 @@ class Molecule(object):
     title = property(_gettitle, _settitle)
     @property
     def unitcell(self):
-        unitcell = self.OBMol.GetData(System.UInt32(ob.openbabelcsharp.UnitCell))
+        unitcell = self.OBMol.GetData(System.UInt32(ob.openbabel_csharp.UnitCell))
         if unitcell:
             return unitcell.Downcast[ob.OBUnitCell]()
         else:
@@ -376,6 +396,14 @@ class Molecule(object):
     def __str__(self):
         return self.write()
 
+    def nodraw(self):
+        """Create 2D coordinates for the molecule."""
+        _operations['gen2D'].Do(self.OBMol)
+        ## Note to self:
+        ## Probably can be changed to be fully featured if desired:
+        ## see http://www.daniweb.com/software-development/python/threads/191210/page11
+        ## for example...
+
     def draw(self, show=True, filename=None, update=False, usecoords=False):
         """Create a 2D depiction of the molecule.
 
@@ -387,118 +415,64 @@ class Molecule(object):
                     (default is False)
           usecoords -- don't calculate 2D coordinates, just use
                        the current coordinates (default is False)
-
-        OASA is used for 2D coordinate generation and depiction. Tkinter and
-        Python Imaging Library are required for image display.
         """
-        etab = ob.OBElementTable()
-
-        if not oasa:
-            errormessage = ("OASA not found, but is required for 2D structure "
-                            "generation and depiction. OASA is part of BKChem. "
-                            "See installation instructions for more "
-                            "information.")
+        if not "png2" in outformats:
+            errormessage = ("PNG output format not found. You should compile "
+                            "Open Babel with PNG support. See installation "
+                            "instructions for more information.")
             raise ImportError(errormessage)
-        mol = oasa.molecule()
-        for atom in self.atoms:
-            v = mol.create_vertex()
-            v.symbol = etab.GetSymbol(atom.atomicnum)
-            v.charge = atom.formalcharge
-            if usecoords:
-                v.x, v.y, v.z = atom.coords[0] * 30., atom.coords[1] * 30., 0.0
-            mol.add_vertex(v)
 
-        for bond in ob.OBMolBondIter(self.OBMol):
-            e = mol.create_edge()
-            e.order = bond.GetBO()
-            if bond.IsHash():
-                e.type = "h"
-            elif bond.IsWedge():
-                e.type = "w"
-            mol.add_edge(bond.GetBeginAtomIdx() - 1,
-                         bond.GetEndAtomIdx() - 1,
-                         e)
-        # I'm sure there's a more elegant way to do the following, but here goes...
-        # let's set the stereochemistry around double bonds
-        self.write("can") # Perceive UP/DOWNness
-        for bond in ob.OBMolBondIter(self.OBMol):
-            ends = bond.GetBeginAtomIdx(), bond.GetEndAtomIdx()
-            if bond.GetBO() == 2:
-                stereobonds = [[b for b in ob.OBAtomBondIter(self.OBMol.GetAtom(x)) if b.GetIdx() != bond.GetIdx() and (b.IsUp() or b.IsDown())]
-                               for x in ends]
-                if stereobonds[0] and stereobonds[1]: # Needs to be defined at either end
-                    if stereobonds[0][0].IsUp() == stereobonds[1][0].IsUp():
-                        # Either both up or both down
-                        stereo = oasa.stereochemistry.cis_trans_stereochemistry.SAME_SIDE
-                    else:
-                        stereo = oasa.stereochemistry.cis_trans_stereochemistry.OPPOSITE_SIDE
-                    atomids = [(b[0].GetBeginAtomIdx(), b[0].GetEndAtomIdx()) for b in stereobonds]
-                    extremes = []
-                    for id, end in zip(ends, atomids):
-                        if end[0] == id:
-                            extremes.append(end[1])
-                        else:
-                            extremes.append(end[0])
-                    center = mol.get_edge_between(mol.atoms[ends[0] - 1], mol.atoms[ends[1] - 1])
-                    st = oasa.stereochemistry.cis_trans_stereochemistry(
-                              center = center, value = stereo,
-                              references = (mol.atoms[extremes[0] - 1], mol.atoms[ends[0] - 1],
-                                            mol.atoms[ends[1] - 1], mol.atoms[extremes[1] - 1]))
-                    mol.add_stereochemistry(st)
-        
-        mol.remove_unimportant_hydrogens()
+        # Need to copy to avoid removing hydrogens from self
+        workingmol = Molecule(ob.OBMol(self.OBMol))
+        workingmol.removeh()
+
         if not usecoords:
-            oasa.coords_generator.calculate_coords(mol, bond_length=30)
-            if update:
-                newcoords = [(v.x / 30., v.y / 30., 0.0) for v in mol.vertices]
-                for atom, newcoord in zip(ob.OBMolAtomIter(self.OBMol), newcoords):
-                    atom.SetVector(*newcoord)
-        if filename or show:
-            maxx = max([v.x for v in mol.vertices])
-            minx = min([v.x for v in mol.vertices])
-            maxy = max([v.y for v in mol.vertices])
-            miny = min([v.y for v in mol.vertices])
-            maxcoord = max(maxx - minx, maxy - miny)
-            fontsize = 16
-            bondwidth = 6
-            linewidth = 2
-            if maxcoord > 270: # 300  - margin * 2
-                for v in mol.vertices:
-                    v.x *= 270. / maxcoord
-                    v.y *= 270. / maxcoord
-                fontsize *= math.sqrt(270. / maxcoord)
-                bondwidth *= math.sqrt(270. / maxcoord)
-                linewidth *= math.sqrt(270. / maxcoord)
-            if filename:
-                filedes = None
+            _operations['gen2D'].Do(workingmol.OBMol)
+        if update == True:
+            if workingmol.OBMol.NumAtoms() != self.OBMol.NumAtoms():
+                errormessage = ("It is not possible to update the original molecule "
+                                "with the calculated coordinates, as the original "
+                                "molecule contains explicit hydrogens for which no "
+                                "coordinates have been calculated.")
+                raise RuntimeError(errormessage)
             else:
-                filedes, filename = tempfile.mkstemp()
+                for i in range(workingmol.OBMol.NumAtoms()):
+                    self.OBMol.GetAtom(i + 1).SetVector(workingmol.OBMol.GetAtom(i + 1).GetVector())
+
+        if filename:
+            workingmol.write("png2", filename=filename, overwrite=True)
+
+        if show:
+            if not filename:
+                errormessage = ("It is only possible to show the molecule if you "
+                                "provide a filename. The reason for this is that I kept "
+                                "having problems when using temporary files.")
+                raise RuntimeError(errormessage)
+                
+            form = _MyForm()
+            form.setup(filename, self.title)
+            Application.Run(form)
             
-            canvas = oasa.cairo_out.cairo_out()
-            canvas.show_hydrogens_on_hetero = True
-            canvas.font_size = fontsize
-            canvas.bond_width = bondwidth
-            canvas.line_width = linewidth
-            canvas.mol_to_cairo(mol, filename)
-            if show:
-                if not tk:
-                    errormessage = ("Tkinter or Python Imaging "
-                                    "Library not found, but is required for image "
-                                    "display. See installation instructions for "
-                                    "more information.")
-                    raise ImportError(errormessage)
-                root = tk.Tk()
-                root.title((hasattr(self, "title") and self.title)
-                           or self.__str__().rstrip())
-                frame = tk.Frame(root, colormap="new", visual='truecolor').pack()
-                image = PIL.open(filename)
-                imagedata = piltk.PhotoImage(image)
-                label = tk.Label(frame, image=imagedata).pack()
-                quitbutton = tk.Button(root, text="Close", command=root.destroy).pack(fill=tk.X)
-                root.mainloop()
-            if filedes:
-                os.close(filedes)
-                os.remove(filename)
+class _MyForm(Form):
+    def __init__(self):
+        Form.__init__(self)
+
+    def setup(self, filename, title):
+        # adjust the form's client area size to the picture
+        self.ClientSize = Size(300, 300)
+        self.Text = title
+         
+        self.filename = filename
+        self.image = Image.FromFile(self.filename)
+        pictureBox = PictureBox()
+        # this will fit the image to the form
+        pictureBox.SizeMode = PictureBoxSizeMode.StretchImage
+        pictureBox.Image = self.image
+        # fit the picture box to the frame
+        pictureBox.Dock = DockStyle.Fill
+         
+        self.Controls.Add(pictureBox)
+        self.Show()
 
 class Atom(object):
     """Represent an Ironable atom.
@@ -672,8 +646,8 @@ class MoleculeData(object):
     def __init__(self, obmol):
         self._mol = obmol
     def _data(self):
-        return [x for x in self._mol.GetData() if x.GetDataType()==ob.openbabelcsharp.PairData
-                                               or x.GetDataType()==ob.openbabelcsharp.CommentData]
+        return [x for x in self._mol.GetData() if x.GetDataType()==ob.openbabel_csharp.PairData
+                                               or x.GetDataType()==ob.openbabel_csharp.CommentData]
     def _testforkey(self, key):
         if not key in self:
             raise KeyError("'%s'" % key)
@@ -718,5 +692,6 @@ class MoleculeData(object):
         return dict(self.iteritems()).__repr__()
  
 if __name__=="__main__": #pragma: no cover
-    import doctest
-    doctest.testmod(verbose=True)
+##    import doctest
+##    doctest.testmod(verbose=True)
+    readstring("smi", "CC(=O)Cl").draw()
