@@ -1,3 +1,10 @@
+## Copyright (c) 2008-2011, Noel O'Boyle
+## All rights reserved.
+##
+##  This file is part of Cinfony.
+##  The contents are covered by the terms of the GPL v2 license
+##  which is included in the file LICENSE_GPLv2.txt.
+
 """
 jybel - A Cinfony module for accessing OpenBabel from Jython
 
@@ -19,12 +26,15 @@ import org.openbabel as ob
 import java.lang.System
 java.lang.System.loadLibrary("openbabel_java")
 
+import javax
+
 def _formatstodict(list):
     list = [list.get(i) for i in range(list.size())]
     broken = [x.replace("[Read-only]", "").replace("[Write-only]","").split(" -- ") for x in list]
     broken = [(x,y.strip()) for x,y in broken]
     return dict(broken)
 _obconv = ob.OBConversion()
+_builder = ob.OBBuilder()
 informats = _formatstodict(_obconv.GetSupportedInputFormat())
 """A dictionary of supported input formats"""
 outformats = _formatstodict(_obconv.GetSupportedOutputFormat())
@@ -33,22 +43,21 @@ outformats = _formatstodict(_obconv.GetSupportedOutputFormat())
 def _getplugins(findplugin, names):
     plugins = dict([(x, findplugin(x)) for x in names if findplugin(x)])
     return plugins
-##def _getpluginnames(ptype): # Doesn't work from Jython!
-##    plugins = ob.vectorString()
-##    ob.OBPlugin.ListAsVector(ptype, None, plugins)
-##    return [x.split()[0] for x in plugins]
+def _getpluginnames(ptype):
+    plugins = ob.vectorString()
+    ob.OBPlugin.ListAsVector(ptype, None, plugins)
+    return [plugins.get(i).split()[0] for i in range(plugins.size())]
 
-descs = ['HBA1', 'HBA2', 'HBD', 'InChI', 'L5', 'logP', 'MR',
-         'MW', 'nF', 'nHal', 's', 'smarts', 'spinMult', 'title', 'TPSA']
+descs = _getpluginnames("descriptors")
 """A list of supported descriptors"""
 _descdict = _getplugins(ob.OBDescriptor.FindType, descs)
-fps = ['FP2', 'FP3', 'FP4', 'MACCS']
+fps = _getpluginnames("fingerprints")
 """A list of supported fingerprint types"""
 _fingerprinters = _getplugins(ob.OBFingerprint.FindFingerprint, fps)
-forcefields = ['ghemical', 'mmff94', 'mmff94s', 'uff']
+forcefields = [_x.lower() for _x in _getpluginnames("forcefields")]
 """A list of supported forcefields"""
 _forcefields = _getplugins(ob.OBForceField.FindType, forcefields)
-operations = ['AddPolarH', 'fillUC', 'gen3D', 'nodative']
+operations = _getpluginnames("ops")
 """A list of supported operations"""
 _operations = _getplugins(ob.OBOp.FindType, operations)
 
@@ -82,12 +91,14 @@ def readfile(format, filename):
         raise ValueError("%s is not a recognised OpenBabel format" % format)
     if not os.path.isfile(filename):
         raise IOError("No such file: '%s'" % filename)
-    obmol = ob.OBMol()
-    notatend = obconversion.ReadFile(obmol,filename)
-    while notatend:
-        yield Molecule(obmol)
+    def filereader():
         obmol = ob.OBMol()
-        notatend = obconversion.Read(obmol)
+        notatend = obconversion.ReadFile(obmol,filename)
+        while notatend:
+            yield Molecule(obmol)
+            obmol = ob.OBMol()
+            notatend = obconversion.Read(obmol)
+    return filereader()
 
 def readstring(format, string):
     """Read in a molecule from a string.
@@ -361,7 +372,7 @@ class Molecule(object):
         to improve the coordinates further.
         """
         forcefield = forcefield.lower()
-        _operations['Gen3D'].Do(self.OBMol)
+        _builder.Build(self.OBMol)
         self.addh()
         self.localopt(forcefield, steps)
 
@@ -376,6 +387,63 @@ class Molecule(object):
     def __str__(self):
         return self.write()
 
+    def draw(self, show=True, filename=None, update=False, usecoords=False):
+        """Create a 2D depiction of the molecule.
+
+        Optional parameters:
+          show -- display on screen (default is True)
+          filename -- write to file (default is None)
+          update -- update the coordinates of the atoms to those
+                    determined by the structure diagram generator
+                    (default is False)
+          usecoords -- don't calculate 2D coordinates, just use
+                       the current coordinates (default is False)
+
+        Tkinter and Python Imaging Library are required for image display.
+        """
+        if not "png2" in outformats:
+            errormessage = ("PNG output format not found. You should compile "
+                            "Open Babel with PNG support. See installation "
+                            "instructions for more information.")
+            raise ImportError(errormessage)
+
+        # Need to copy to avoid removing hydrogens from self
+        workingmol = Molecule(ob.OBMol(self.OBMol))
+        workingmol.removeh()
+
+        if not usecoords:
+            _operations['gen2D'].Do(workingmol.OBMol)
+        if update == True:
+            if workingmol.OBMol.NumAtoms() != self.OBMol.NumAtoms():
+                errormessage = ("It is not possible to update the original molecule "
+                                "with the calculated coordinates, as the original "
+                                "molecule contains explicit hydrogens for which no "
+                                "coordinates have been calculated.")
+                raise RuntimeError(errormessage)
+            else:
+                for i in range(workingmol.OBMol.NumAtoms()):
+                    self.OBMol.GetAtom(i + 1).SetVector(workingmol.OBMol.GetAtom(i + 1).GetVector())
+
+        if filename:
+            filedes = None
+        else:
+            filedes, filename = tempfile.mkstemp()
+        
+        workingmol.write("png2", filename=filename, overwrite=True)
+        
+        if show:
+            image = javax.imageio.ImageIO.read(java.io.File(filename))
+            frame = javax.swing.JFrame(visible=1)
+            frame.getContentPane().add(javax.swing.JLabel(javax.swing.ImageIcon(image)))
+            frame.setSize(300,300)
+            frame.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE)
+            frame.show()
+            
+        if filedes:
+            os.close(filedes)
+            os.remove(filename)
+
+
 class Atom(object):
     """Represent a Jybel atom.
 
