@@ -1,3 +1,10 @@
+## Copyright (c) 2008-2011, Noel O'Boyle
+## All rights reserved.
+##
+##  This file is part of Cinfony.
+##  The contents are covered by the terms of the BSD license
+##  which is included in the file LICENSE_BSD.txt.
+
 """
 rdkit - A Cinfony module for accessing the RDKit from CPython
 
@@ -13,9 +20,10 @@ Global variables:
 import os
 
 from rdkit import Chem
-from rdkit.Chem import AllChem
-from rdkit.Chem.Draw import MolDrawing
-from rdkit.Chem.AvailDescriptors import descDict
+from rdkit.Chem import AllChem, Draw
+from rdkit.Chem import Descriptors
+
+_descDict = dict(Descriptors.descList)
 
 import rdkit.DataStructs
 import rdkit.Chem.MACCSkeys
@@ -33,19 +41,22 @@ except:
 # Aggdraw
 try:
     import aggdraw
+    from rdkit.Chem.Draw import aggCanvas
 except ImportError:
     aggdraw = None
 
 fps = ['rdkit', 'layered', 'maccs', 'atompairs', 'torsions']
 """A list of supported fingerprint types"""
-descs = descDict.keys()
+descs = _descDict.keys()
 """A list of supported descriptors"""
 
-_formats = {'smi': "SMILES", 'iso': "Isomeric SMILES",
+_formats = {'smi': "SMILES",
+            'can': "Canonical SMILES",
             'mol': "MDL MOL file", 'sdf': "MDL SDF file"}
 informats = dict([(_x, _formats[_x]) for _x in ['mol', 'sdf', 'smi']])
 """A dictionary of supported input formats"""
-outformats = dict([(_x, _formats[_x]) for _x in ['mol', 'sdf', 'smi', 'iso']])
+outformats = dict([(_x, _formats[_x]) for _x in ['mol', 'can', 'sdf',
+                                                 'smi']])
 """A dictionary of supported output formats"""
 
 _forcefields = {'uff': AllChem.UFFOptimizeMolecule}
@@ -153,7 +164,7 @@ class Outputfile(object):
         if format=="sdf":
             self._writer = Chem.SDWriter(self.filename)
         elif format=="smi":
-            self._writer = Chem.SmilesWriter(self.filename)
+            self._writer = Chem.SmilesWriter(self.filename, isomericSmiles=True)
         else:
             raise ValueError,"%s is not a recognised RDKit format" % format
         self.total = 0 # The total number of molecules written to the file
@@ -183,7 +194,7 @@ class Molecule(object):
        Mol -- an RDKit Mol or any type of cinfony Molecule
       
     Attributes:
-       atoms, data, molwt, title
+       atoms, data, formula, molwt, title
     
     Methods:
        addh(), calcfp(), calcdesc(), draw(), localopt(), make3D(), removeh(),
@@ -210,7 +221,9 @@ class Molecule(object):
     @property
     def data(self): return MoleculeData(self.Mol)
     @property
-    def molwt(self): return descDict['MolWt'](self.Mol)
+    def molwt(self): return Descriptors.MolWt(self.Mol)
+    @property
+    def formula(self): return Descriptors.MolecularFormula(self.Mol)    
     def _gettitle(self):
         # Note to self: maybe should implement the get() method for self.data
         if "_Name" in self.data:
@@ -222,7 +235,7 @@ class Molecule(object):
     @property
     def _exchange(self):
         if self.Mol.GetNumConformers() == 0:
-            return (0, self.write("iso"))
+            return (0, self.write("smi"))
         else:
             return (1, self.write("mol"))
 
@@ -255,9 +268,9 @@ class Molecule(object):
             if not overwrite and os.path.isfile(filename):
                 raise IOError, "%s already exists. Use 'overwrite=True' to overwrite it." % filename
         if format=="smi":
-            result = Chem.MolToSmiles(self.Mol)
-        elif format=="iso":
-            result = Chem.MolToSmiles(self.Mol, isomericSmiles=True)
+            result = Chem.MolToSmiles(self.Mol, isomericSmiles=True, canonical=False)
+        elif format=="can":
+            result = Chem.MolToSmiles(self.Mol, isomericSmiles=True, canonical=True)
         elif format=="mol":
             result = Chem.MolToMolBlock(self.Mol)
         else:
@@ -294,7 +307,7 @@ class Molecule(object):
         ans = {}
         for descname in descnames:
             try:
-                desc = descDict[descname]
+                desc = _descDict[descname]
             except KeyError:
                 raise ValueError, "%s is not a recognised RDKit descriptor type" % descname
             ans[descname] = desc(self.Mol)
@@ -337,54 +350,34 @@ class Molecule(object):
           usecoords -- don't calculate 2D coordinates, just use
                        the current coordinates (default is False)
 
-        Aggdraw is used for 2D depiction. Tkinter and
+        Aggdraw or Cairo is used for 2D depiction. Tkinter and
         Python Imaging Library are required for image display.
         """
-        if usecoords:
-            confId = 0
-        else:
-            if update:
-                AllChem.Compute2DCoords(self.Mol)
-                confId = 0
-            else:
-                confId = self.Mol.GetNumConformers()
-                AllChem.Compute2DCoords(self.Mol, clearConfs = False)
-        if show or filename:
-            if not aggdraw:
-                errormessage = ("Aggdraw not found, but is required for 2D"
-                                "structure depiction. "
-                                "See installation instructions for more "
-                                "information.")
-                raise ImportError, errormessage
-              
-            Chem.Kekulize(self.Mol)
-            MolDrawing.registerCanvas('agg')
-            img = PIL.new("RGBA",(300,300),"white")
-            canvas = aggdraw.Draw(img)                
-            canvas.setantialias(True)
-            drawer = MolDrawing.MolDrawing(canvas)
-            drawer.wedgeDashedBonds = True
-            drawer.AddMol(self.Mol, confId = confId)
-            canvas.flush()
-        
-            if filename: # Note: overwrite is allowed          
-                img.save(filename)
-            if show:
-                if not tk:
-                    errormessage = ("Tkinter or Python Imaging "
-                                    "Library not found, but is required for image "
-                                    "display. See installation instructions for "
-                                    "more information.")
-                    raise ImportError, errormessage                  
-                root = tk.Tk()
-                root.title((hasattr(self, "title") and self.title)
-                           or self.__str__().rstrip())
-                frame = tk.Frame(root, colormap="new", visual='truecolor').pack()
-                imagedata = PILtk.PhotoImage(img)
-                label = tk.Label(frame, image=imagedata).pack()
-                quitbutton = tk.Button(root, text="Close", command=root.destroy).pack(fill=tk.X)
-                root.mainloop()
-            Chem.SanitizeMol(self.Mol)
+        if not usecoords and update:
+            AllChem.Compute2DCoords(self.Mol)
+            usecoords = True
+        mol = Chem.Mol(self.Mol.ToBinary()) # Clone
+        if not usecoords:
+            AllChem.Compute2DCoords(mol)
+               
+        if filename: # Note: overwrite is allowed          
+            Draw.MolToFile(mol, filename)
+        if show:
+            if not tk:
+                errormessage = ("Tkinter or Python Imaging "
+                                "Library not found, but is required for image "
+                                "display. See installation instructions for "
+                                "more information.")
+                raise ImportError(errormessage)
+            img = Draw.MolToImage(mol)
+            root = tk.Tk()
+            root.title((hasattr(self, "title") and self.title)
+                       or self.__str__().rstrip())
+            frame = tk.Frame(root, colormap="new", visual='truecolor').pack()
+            imagedata = PILtk.PhotoImage(img)
+            label = tk.Label(frame, image=imagedata).pack()
+            quitbutton = tk.Button(root, text="Close", command=root.destroy).pack(fill=tk.X)
+            root.mainloop()
 
     def localopt(self, forcefield = "uff", steps = 500):
         """Locally optimize the coordinates.
