@@ -3,30 +3,50 @@ import os
 import sys
 import unittest
 
-ironable = rdk = cdk = obabel = webel = None
+pybel = indy = ironable = rdk = cdk = obabel = webel = opsin = None
+try:
+    import pybel # From Open Babel
+except ImportError:
+    pass
 try:
     from cinfony import cdk
-except (ImportError, KeyError):
+except (RuntimeError, ImportError, KeyError):
     pass
 try:
     from cinfony import obabel
-except (ImportError, AttributeError):
+except (ImportError, AttributeError, KeyError):
     pass
 try:
     from cinfony import rdk
 except ImportError:
     pass
-from cinfony import webel
+try:
+    from cinfony import opsin
+except (ImportError, KeyError):
+    pass
+try:
+    from cinfony import indy
+except (IOError, ImportError, KeyError):
+    pass
+try:
+    from cinfony import webel
+except ImportError:
+    pass
+
+try: # Define next() for Jython 2.5
+    next
+except (NameError):
+    next = lambda x: x.next()
 
 class myTestCase(unittest.TestCase):
     """Additional methods not present in Jython 2.2"""
     # Taken from unittest.py in Python 2.5 distribution
     def assertFalse(self, expr, msg=None):
         "Fail the test if the expression is true."
-        if expr: raise self.failureException, msg
+        if expr: raise self.failureException(msg)
     def assertTrue(self, expr, msg=None):
         """Fail the test unless the expression is true."""
-        if not expr: raise self.failureException, msg
+        if not expr: raise self.failureException(msg)
     def assertAlmostEqual(self, first, second, places=7, msg=None):
         """Fail if the two objects are unequal as determined by their
            difference rounded to the given number of decimal places
@@ -36,8 +56,34 @@ class myTestCase(unittest.TestCase):
            as significant digits (measured from the most signficant digit).
         """
         if round(second-first, places) != 0:
-            raise self.failureException, \
-                  (msg or '%r != %r within %r places' % (first, second, places))
+            raise self.failureException(
+                  (msg or '%r != %r within %r places' % (first, second, places)))
+
+class TestOpsin(myTestCase):
+    toolkit = opsin
+    
+    def testconversion(self):
+        """Convert from acetylsaliclyic acid to other formats"""
+        mol = self.toolkit.readstring("iupac", "benzene")
+        self.assertEqual(mol.write("smi"), "C1=CC=CC=C1")
+        self.assertEqual(mol.write("inchi"), "InChI=1/C6H6/c1-2-4-6-5-3-1/h1-6H")
+        cml = mol.write("cml")
+    def testnoconversion(self):
+        """A failed conversion - should raise IOError"""
+        self.assertRaises(IOError, self.toolkit.readstring, "iupac", "Nosuchname")
+    def testnoformats(self):
+        """No such format - should raise ValueError"""
+        self.assertRaises(ValueError, self.toolkit.readstring, "noel", "benzene")
+    def testwritefile(self):
+        """Test writing a file"""
+        if os.path.isfile("tmp.cml"):
+            os.remove("tmp.cml")
+        mol = self.toolkit.readstring("iupac", "benzene")
+        mol.write("cml", "tmp.cml")
+        self.assertTrue(os.path.isfile("tmp.cml"))
+        self.assertRaises(IOError, mol.write, "cml", "tmp.cml")
+        mol.write("cml", "tmp.cml", overwrite=True)
+        os.remove("tmp.cml")
 
 class TestToolkit(myTestCase):
     
@@ -76,7 +122,7 @@ class TestToolkit(myTestCase):
     def testFPbits(self):
         """Test whether the bits are set correctly."""
         bits = [x.calcfp().bits for x in self.mols]
-        self.assertEqual(len(bits[0]), self.Nbits)
+        self.assertNotEqual(len(bits[0]), 0)
         bits = [set(x) for x in bits]
         # Calculate the Tanimoto coefficient the old-fashioned way
         tanimoto = len(bits[0] & bits[1]) / float(len(bits[0] | bits[1]))
@@ -168,14 +214,14 @@ M  END
         self.assertEqual(len(self.mols), 2)
 
     def RFreaderror(self):
-        mol = self.toolkit.readfile("sdf", "nosuchfile.sdf").next()
+        mol = next(self.toolkit.readfile("sdf", "nosuchfile.sdf"))
 
     def testRFmissingfile(self):
         """Test that reading from a non-existent file raises an error."""
         self.assertRaises(IOError, self.RFreaderror)
 
     def RFformaterror(self):
-        mol = self.toolkit.readfile("noel", "head.sdf").next()
+        mol = next(self.toolkit.readfile("noel", "head.sdf"))
     
     def testRFformaterror(self):
         """Test that invalid formats raise an error"""
@@ -312,9 +358,8 @@ class TestOBabel(TestToolkit):
     tanimotoresult = 1/3.
     Natoms = 15
     tpsaname = "TPSA"
-    Nbits = 3
     Nfpbits = 32
-    datakeys = ['NSC', 'Comment']
+    datakeys = ['NSC', 'Comment', 'OpenBabel Symmetry Classes']
 
     def testFP_FP3(self):
         "Checking the results from FP3"
@@ -323,7 +368,7 @@ class TestOBabel(TestToolkit):
 
     def testunitcell(self):
         """Testing unit cell access"""
-        mol = self.toolkit.readfile("cif", "hashizume.cif").next()
+        mol = next(self.toolkit.readfile("cif", "hashizume.cif"))
         cell = mol.unitcell
         self.assertAlmostEqual(cell.GetAlpha(), 92.9, 1)
 
@@ -338,21 +383,6 @@ class TestOBabel(TestToolkit):
     def importtest(self):
         self.mols[0].draw(show=True, usecoords=True)
 
-##    def testDrawdependencies(self):
-##        "Testing the draw dependencies"
-##        t = self.toolkit.tk
-##        self.toolkit.tk = None
-##        self.mols[0].draw(show=False, usecoords=True,
-##                          filename="%s_b.png" % self.toolkit.__name__)
-##        self.assertRaises(ImportError,
-##                          self.importtest)
-##        self.toolkit.tk = t
-##
-##        t = self.toolkit.oasa
-##        self.toolkit.oasa = None
-##        self.assertRaises(ImportError,
-##                          self.importtest)
-##
     def testRSgetprops(self):
         """Get the values of the properties."""
         self.assertAlmostEqual(self.mols[0].exactmass, 58.078, 3)
@@ -361,23 +391,21 @@ class TestOBabel(TestToolkit):
         self.assertRaises(AttributeError, self.RSaccesstest)
 
 class TestJybel(TestOBabel):
-    def testDrawdependencies(self):
-        "No testing the draw dependencies"
-        pass
+    pass
 
+class TestIronable(TestJybel):
     def testDraw(self):
         """No creating a 2D depiction"""
         pass
 
-class TestIronable(TestJybel):
-    pass
+class TestPybel(TestOBabel):
+    toolkit = pybel
 
 class TestRDKit(TestToolkit):
     toolkit = rdk
     tanimotoresult = 1/3.
     Natoms = 9
     tpsaname = "TPSA"
-    Nbits = 12
     Nfpbits = 64
     datakeys = ['NSC']
 
@@ -385,12 +413,39 @@ class TestRDKit(TestToolkit):
         """No conversion to MOL2 done"""
         pass
 
+class TestIndigo(TestToolkit):
+    toolkit = indy
+    tanimotoresult = 1/3.
+    Natoms = 15
+    tpsaname = "TPSA"
+    Nfpbits = 934
+    datakeys = ['NSC']
+
+    def testRSconversiontoMOL2(self):
+        """No conversion to MOL2 done"""
+        pass
+
+    def testRFdesc(self):
+        """No descriptors"""
+        pass
+
+    def testattributes(self):
+        """Test attributes like informats, descs and so on"""
+        informats, outformats = self.toolkit.informats, self.toolkit.outformats
+        self.assertNotEqual(len(self.toolkit.informats.keys()), 0)
+        self.assertNotEqual(len(self.toolkit.outformats.keys()), 0)
+        self.assertNotEqual(len(self.toolkit.fps), 0)
+
+    def testLocalOpt(self):
+        """No forcefields"""
+        pass
+
+
 class TestWebel(TestToolkit):
     toolkit = webel
     tanimotoresult = 0.375
     Natoms = 9
     tpsaname = "TPSADescriptor_TopoPSA"
-    Nbits = 4
     Nfpbits = 1
     datakeys = ['NSC']
 
@@ -512,10 +567,9 @@ $$$$"""
         
 class TestCDK(TestToolkit):
     toolkit = cdk
-    tanimotoresult = 0.571
+    tanimotoresult = 0.375
     Natoms = 15
     tpsaname = "tpsa"
-    Nbits = 4
     Nfpbits = 4 # The CDK uses a true java.util.Bitset
     datakeys = ['NSC', 'cdk:Remark', 'cdk:Title']
 
@@ -534,15 +588,18 @@ class TestCDK(TestToolkit):
         self.assertEqual(len(self.mols[0].atoms), 4)
         self.assertRaises(AttributeError, self.RSaccesstest)
 
-##    def testRFoutputfile(self):
-##        pass
+class TestCDKJPype(TestCDK):
+    def testDraw(self):
+        """No depiction supported I'm afraid"""
+        pass
 
 if __name__=="__main__":
     if os.path.isfile("testoutput.txt"):
         os.remove("testoutput.txt")
 
     lookup = {'cdk': TestCDK, 'obabel':TestOBabel, 'rdk':TestRDKit,
-              'webel': TestWebel}
+              'webel': TestWebel, 'opsin': TestOpsin, 'indy': TestIndigo,
+              'pybel':TestPybel}
     if sys.platform[:4] == "java":
         lookup['obabel'] = TestJybel
         del lookup['rdk']
@@ -550,13 +607,17 @@ if __name__=="__main__":
         lookup['obabel'] = TestIronable
         del lookup['rdk']
         del lookup['cdk']
+        del lookup['opsin']
+    else:
+        lookup['cdk'] = TestCDKJPype
 
-    testcases = lookup.values()
+    # Only run Pybel tests if specifically asked
+    testcases = list(lookup.values()).remove(TestPybel)
 
     if len(sys.argv) > 1:
         testcases = [lookup[x] for x in sys.argv[1:]]
 
     for testcase in testcases:
-        print "\n\n\nTESTING %s\n%s\n\n" % (testcase.__name__, "== "*10)
+        print("\n\n\nTESTING %s\n%s\n\n" % (testcase.__name__, "== "*10))
         myunittest = unittest.defaultTestLoader.loadTestsFromTestCase(testcase)
-        unittest.TextTestRunner(verbosity=2).run(myunittest)
+        unittest.TextTestRunner(verbosity=1).run(myunittest)
